#!/bin/sh
#
# Autoplay methods encoder script
#
# Copyright 2018 Damian Yerrick
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
set -e

# Encodes we want to do, all at 256x144, 15 fps, 8 seconds:
# WebM, AVC, GIF, JPEG sequence, PNG sequence,
# JPEG filmstrip, PNG filmstrip
# Display routines:
# WebM, AVC, GIF, sequence JS, filmstrip JS, filmstrip CSS
# Possibly even sequence CSS, using animated z-index or opacity
mkdir -p encoded

# Null 
# If using "scale=240:-1" instead of "crop=192:144",
# scale height -1 and -2 mean closest integer and closest even
# integer to the ideal aspect ratio.  (Even is needed for
# pix_fmt that use 4:2:0 chroma sampling but not for GIF.)
ffmpeg -y -ss 35.0 -t 8.0 -i vidsrc/vidstab.mp4 -an \
  -pix_fmt yuv420p -profile:v baseline -c:v libx264 \
  -filter_complex "[0:v] fps=15,crop=192:144" \
  encoded/scooting.mp4

# libvpx uses -b:v as the maximum bitrate, so it has to be made
# arbitrarily large so that -crf can do its job
ffmpeg -y -ss 35.0 -t 8.0 -i vidsrc/vidstab.mp4 -an \
  -pix_fmt yuv420p \
  -filter_complex "[0:v] fps=15,crop=192:144" \
  -c:v libvpx -crf 30 -b:v 1M \
  encoded/scooting.webm
ffmpeg -y -ss 35.0 -t 8.0 -i vidsrc/vidstab.mp4 -an \
  -pix_fmt yuv420p \
  -filter_complex "[0:v] fps=15,crop=192:144" \
  -c:v libvpx-vp9 -crf 30 -b:v 1M \
  encoded/scooting-vp9.webm

# https://superuser.com/a/556031/302629
# https://engineering.giphy.com/how-to-make-gifs-with-ffmpeg/
# ffmpeg 2 or later can make a reasonable GIF palette
# but it's better to do it in 2 passes than with the 1-liner because
# split into palettegen needs to buffer the entire video in a FIFO,
# which eats RAM.
# https://superuser.com/a/1135202/302629
ffmpeg -y -ss 35.0 -t 8.0 -i vidsrc/vidstab.mp4 -an \
  -filter_complex "[0:v] fps=15,crop=192:144,palettegen=127" \
  encoded/palette.png
ffmpeg -y -ss 35.0 -t 8.0 -i vidsrc/vidstab.mp4 -i encoded/palette.png -an \
  -filter_complex "[0:v] fps=15,crop=192:144 [b]; [b][1:v] paletteuse=bayer:4" \
  encoded/scooting.gif

# Make a filmstrip for use with CSS MJPEG (sprites with stepped animation)
# https://superuser.com/a/984688/302629
# An alternative to fps=15 is select='not(mod(n,30))'
ffmpeg -y -ss 35.0 -t 8.0 -i vidsrc/vidstab.mp4 -an \
  -filter_complex "fps=15,crop=192:144,tile=layout=1x120" \
  -vframes 1 -q:v 4 encoded/scooting.jpg
ffmpeg -y -ss 35.0 -t 8.0 -i vidsrc/vidstab.mp4 -i encoded/palette.png -an \
  -filter_complex "[0:v] fps=15,crop=192:144,tile=layout=1x120 [b]; [b][1:v] paletteuse=bayer:4" \
  -vframes 1 encoded/scooting.png

# Make a series of images for JS rotation
ffmpeg -y -ss 35.0 -t 8.0 -i vidsrc/vidstab.mp4 -an \
  -filter_complex "fps=15,crop=192:144" \
  -vframes 120 -q:v 4 encoded/scooting-%04d.jpg
ffmpeg -y -ss 35.0 -t 8.0 -i vidsrc/vidstab.mp4 -i encoded/palette.png -an \
  -filter_complex "[0:v] fps=15,crop=192:144 [b]; [b][1:v] paletteuse=bayer:4" \
  -vframes 120 encoded/scooting-%04d.png

# Use AdvanceCOMP's advpng to recompress all images, saving about 7%
advpng -z3 encoded/scooting.png 
advpng -z3 encoded/scooting-*.png

# The source code for the encode script
zip -9 enc.sh.zip enc.sh
